from PySide.QtCore import *
from PySide.QtGui  import *

from .notmuch_cli import *

class ThreadSummary(QObject):
    def __init__(self, line):
        line = line.lstrip("[")
        line = line.rstrip("\n")
        line = line.rstrip(",")
        line = line.rstrip("]")
        
        self._data = json.loads(line)
        self.date = datetime.fromtimestamp(self.timestamp)
        self.id   = self._data["thread"]
        self.tags = " ".join(self._data["tags"])
        self.color = "white"
        
        if "unread" in self._data["tags"]:
            self.color = "gray"
    
    def __getattr__(self, key):
        return self._data[key]
    
    def __str__(self):
        return json.dumps(self._data)

class SearchModel(QAbstractListModel):
    SUBJECT_ROLE = Qt.UserRole + 1
    DATE_ROLE    = Qt.UserRole + 2
    AUTHORS_ROLE = Qt.UserRole + 3
    ID_ROLE      = Qt.UserRole + 4
    TAGS_ROLE    = Qt.UserRole + 5
    COLOR_ROLE   = Qt.UserRole + 6

    def __init__(self, parent=None):
        super(SearchModel, self).__init__(parent)
        self._data = []
        
        keys = {}
        keys[self.SUBJECT_ROLE] = 'subject'
        keys[self.DATE_ROLE   ] = 'date'
        keys[self.AUTHORS_ROLE] = 'authors'
        keys[self.ID_ROLE     ] = 'id'
        keys[self.TAGS_ROLE   ] = 'tags'
        keys[self.COLOR_ROLE  ] = 'thread_color'
        self.setRoleNames(keys)
    
    def rowCount(self, index):
        return len(self._data)
    
    def data(self, index, role):
        if not index.isValid():
            return None
        
        if index.row() > len(self._data):
            return None
        
        line = self._data[index.row()]
        
        if role == self.SUBJECT_ROLE:
            return line.subject
        elif role == self.DATE_ROLE:
            return str(line.date)
        elif role == self.AUTHORS_ROLE:
            return line.authors
        elif role == self.ID_ROLE:
            return line.id
        elif role == self.TAGS_ROLE:
            return line.tags
        elif role == self.COLOR_ROLE:
            return line.color
        else:
            return None
    
    @Slot(unicode)
    def search(self, query):
        search = Search(query)
        search.thread.connect(self.add)
        search.start()
    
    @Slot(unicode)
    def add(self, line):
        count = len(self._data)
        self.beginInsertRows(QModelIndex(), count, count)
        self._data.append(ThreadSummary(line))
        self.endInsertRows()

